/***************************************************************************
 *   Copyright (C) 2004 by Hans Oischinger                                 *
 *   hans.oischinger@kdemail.net                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA 02110-1301, USA.             *
 ***************************************************************************/
#include "generictool.h"

// KDE
#include <klocale.h>

GenericTool::GenericTool(QObject *parent, const char *name)
    : QObject(parent, name)
{
  m_mouseButtonIsDown = false;
  m_isDragging = false;
  m_shiftPressed = false;
  m_ctrlPressed = false;
  m_altPressed = false;
}


GenericTool::~GenericTool()
{}

bool GenericTool::mouseEvent( QMouseEvent* mouseEvent, const QPoint &canvasCoordinate )
{
  m_lastPoint.setX( canvasCoordinate.x() );
  m_lastPoint.setY( canvasCoordinate.y() );

  setCursor();

  m_altPressed = mouseEvent->state() & Qt::AltButton;

  // Mouse events:
  if( mouseEvent->type() == QEvent::MouseButtonDblClick )
  {
    mouseButtonDblClick();

    return true;
  }

  if( mouseEvent->type() == QEvent::MouseButtonPress )
  {
    m_firstPoint.setX( canvasCoordinate.x() );
    m_firstPoint.setY( canvasCoordinate.y() );

    mouseButtonPress();

    m_mouseButtonIsDown = true;

    return true;
  }

  if( mouseEvent->type() == QEvent::MouseMove )
  {
    //setCursor();

    if( m_mouseButtonIsDown )
    {
      mouseDrag();

      m_isDragging = true;
    }
    else
      mouseMove();

    return true;
  }

  if( mouseEvent->type() == QEvent::MouseButtonRelease )
  {
    if( m_isDragging )
    {
      mouseDragRelease();

      m_isDragging = false;
    }
    else if( m_mouseButtonIsDown )	// False if canceled.
      mouseButtonRelease();

    m_mouseButtonIsDown = false;

    return true;
  }

  return false;
}

bool
GenericTool::keyEvent( QEvent* event )
{
  // Key press events.
  if( event->type() == QEvent::KeyPress )
  {
    QKeyEvent* keyEvent = static_cast<QKeyEvent*>( event );

    // Terminate the current drawing with the Enter-key:
    if(
      ( keyEvent->key() == Qt::Key_Enter ||
        keyEvent->key() == Qt::Key_Return )
      && !m_isDragging )
    {
      accept();

      return true;
    }

    // Terminate the current drawing with the Enter-key:
    if( keyEvent->key() == Qt::Key_Backspace && !m_isDragging )
    {
      cancelStep();

      return true;
    }

    // Cancel dragging with ESC-key:
    if( keyEvent->key() == Qt::Key_Escape )
    {
      cancel();

      m_isDragging = false;
      m_mouseButtonIsDown = false;

      return true;
    }

    // If SHIFT is pressed, some tools create a "square" object while dragging:
    if( keyEvent->key() == Qt::Key_Shift )
    {
      m_shiftPressed = true;
      if( m_isDragging )
      {
        mouseDragShiftPressed();

        return true;
      }
    }

    // If Ctrl is pressed, some tools create a "centered" object while dragging:
    if( keyEvent->key() == Qt::Key_Control )
    {
      m_ctrlPressed = true;
      if( m_isDragging )
      {
        mouseDragCtrlPressed();

        return true;
      }
    }

  }

  // Key release events:
  if( event->type() == QEvent::KeyRelease )
  {
    QKeyEvent* keyEvent = static_cast<QKeyEvent*>( event );

    Qt::Key key = (Qt::Key)keyEvent->key();
    if( key == Qt::Key_Shift )
    {
      m_shiftPressed = false;
      if( m_isDragging )
      {
        mouseDragShiftReleased();

        return true;
      }
    }

    if( key == Qt::Key_Control )
    {
      m_ctrlPressed = false;
      if( m_isDragging )
      {
        mouseDragCtrlReleased();

        return true;
      }
    }

    if( key == Qt::Key_Left || key == Qt::Key_Right || key == Qt::Key_Up || key == Qt::Key_Down )
    {
      arrowKeyReleased( key );
      return true;
    }

    return keyReleased( key );
  }

  return false;
}

QString GenericTool::name( ) const
{
  return i18n( "Unnamed Tool" );
}

QString GenericTool::contextHelp( ) const
{
  return i18n( "This tool has no description." );
}


#include "generictool.moc"
