/*
Copyright 2008  Nathan Bradshaw nathanlbradshaw@gmail.com

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) version 3, or any
later version accepted by the membership of KDE e.V. (or its
successor approved by the membership of KDE e.V.), which shall
act as a proxy defined in Section 6 of version 3 of the license.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public 
License along with this library.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef QUERYMANAGER_H
#define QUERYMANAGER_H

#include <QtCore/QObject>
//#include "kjob.h"
//#include "query.h"
//#include "service.h"

#include "servicemodel_export.h"

// Namespace
namespace ServiceModel { 
    
    class Query;
    class Service;
    class QueryManagerPrivate;
    
    /** Manages query queues for each service a client wishes torun  queries against. Dispatches queries and creates results */
    class SERVICEMODEL_EXPORT QueryManager : public QObject{
        Q_OBJECT
        
        public:
            QueryManager (QObject *parent = 0);
            virtual ~QueryManager ( );
            
            int addQuery (Query *query, int serviceId);
            
            int createServiceQueue (Service *service, int serviceId);
            
            void startQueue (int serviceId);
            
            /** pause queue processing for a service */
            void pauseQueue (int serviceId) ;
            
            int queueLenth (int serviceId) const;
            
            bool isProcessing (int serviceId) const;
            
        protected:
            /** dynamic pointer for private class */
            QueryManagerPrivate * const d_ptr;
            
            /** Constructor */
            QueryManager(QueryManagerPrivate &dd, QObject *parent);
            
        private:
            Q_DECLARE_PRIVATE(QueryManager)
            
    };
} 
#endif //QUERYMANAGER_H

