/*  This file is part of the KDE project
    Copyright (C) 2007 Matthias Kretz <kretz@kde.org>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License version 2 as published by the Free Software Foundation.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.

*/

#ifndef PHONON_MIXERENUMCONTROL_H
#define PHONON_MIXERENUMCONTROL_H

#include "phonon_mixer_export.h"
#include "mixercontrol.h"

class QStringList;
class QString;

namespace Phonon
{
namespace Mixer
{

class MixerEnumControlPrivate;
class PHONON_MIXER_EXPORT MixerEnumControl : public MixerControl
{
    friend class Mixer;
    friend class MixerSharedData;
    friend MixerEnumControl mixercontrol_cast<MixerEnumControl>(const MixerControl &mc);
    friend MixerEnumControl qobject_cast<MixerEnumControl>(QObject *o);
    Q_DECLARE_PRIVATE(MixerEnumControl)
    public:
        enum Category {
            CaptureSource
        };
        MixerEnumControl(const MixerEnumControl &);
        MixerEnumControl &operator=(const MixerEnumControl &);

        QStringList options() const;
        QString currentOption() const;
        int currentIndex() const;

        Category category() const;

#if 0 // doxygen should pick this up
    Q_SIGNALS:
        void currentIndexChanged(int);
        void currentIndexChanged(const QString &);

    public Q_SLOTS:
#endif
        void setCurrentIndex(const QString &);
        void setCurrentIndex(int);

    protected:
        MixerEnumControl(MixerEnumControlPrivate &);
};

template<> PHONON_MIXER_EXPORT MixerEnumControl mixercontrol_cast<MixerEnumControl>(const MixerControl &mc);

} // namespace Mixer
} // namespace Phonon

template<> PHONON_MIXER_EXPORT Phonon::Mixer::MixerEnumControl qobject_cast<Phonon::Mixer::MixerEnumControl>(QObject *o);

#endif // PHONON_MIXERENUMCONTROL_H
