/*  This file is part of the KDE project
    Copyright (C) 2007 Matthias Kretz <kretz@kde.org>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) version 3.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.

*/

#include "effect.h"
#include <QtCore/QList>

namespace Phonon
{
namespace Proxy
{

Effect::Effect(int effectId, QObject *parent)
    : QObject(parent)
{
    createProxiedObject(BackendInterface::EffectClass, QVariantList() << QVariant(effectId));
}

Effect::~Effect()
{
    send(Destructor);
    receive(Destructor);
}

QList<EffectParameter> Effect::parameters() const
{
    send(EffectParameters);
    QList<EffectParameter> r;
    receive(EffectParameters) >> r;
    return r;
}

QVariant Effect::parameterValue(const EffectParameter &p) const
{
    send(EffectParameterValue) << p;
    QVariant r;
    receive(EffectParameterValue) >> r;
    return r;
}

void Effect::setParameterValue(const EffectParameter &p, const QVariant &newValue)
{
    send(EffectSetParameterValue) << p << newValue;
    receive(EffectSetParameterValue);
}

}} //namespace Phonon::Proxy
